// -*- mode:C++; tab-width: 4 -*-
/*
    Multi-threaded C version fo email-fmt.py.
    Copyright (C) 2005 Grzegorz Adam Hankiewicz.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
    by the Free Software Foundation; either version 2 of the License,
    or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
    USA.
*/
#include "email-fmt.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>


/**[txh]********************************************************************

  Description: Shows a small bit of commandline help documenting options.
  
***************************************************************************/

static void show_help(void)
{
    printf("email-fmt, by Grzegorz Adam Hankiewicz. This is GPL software.\n\n"

"email-fmt requires the GNU fmt tool to be present on your system in\n"
"order to work properly. email-fmt parses email alike text from STDIN,\n"
"detects blocks of indentation, reformats them to the specified width,\n"
"and writes them to STDOUT respecting indentation levels. Usage:\n\n"

" cat text | email-fmt [-h] [-soft] [-width number] [-indentation chars]\n\n"

" -h\n"
"    Shows this help\n"
" -soft\n"
"    Whitespace indentation is significant during block detection.\n"
" -width number\n"
"    Text will be formatted to `number' colums, if it is bigger\n"
"    than zero.  The default is 70 columns.\n"
" -indentation chars\n"
"    The string `chars' should contain the characters this wrapper\n"
"    detects as indentation markers. Usually things like `>',\n"
"    ` ' or `:'. You may have to quote them from the shell.\n");
}

/**[txh]********************************************************************

  Description: Main entry of the binary. Nothing special, parses the command
  line and then calls the real process.
  
  Return: 0 on success, non zero on error.
  
***************************************************************************/

int main(int argc, char *argv[])
{
    int soft = 0, width = 70;
    char *indentation = 0;

    /* Parse the commandline. */
    for (int f = 0; f < argc; f++) {
        /* Search for softness boolean parameter. */
        if (!strcmp(argv[f], "-soft"))
            soft++;
        /* Maybe some help? */
        else if (!strncmp(argv[f], "-h", 2)) {
            show_help();
            return 0;
        }
        /* If there are enough parameters, parse those with options. */
        else if (f < argc - 1) {
            /* Look for a specific width. */
            if (!strcmp(argv[f], "-width")) {
                width = strtol(argv[f + 1], NULL, 10);
                f++;
            }
            /* Look for indentation characters. */
            else if (!strcmp(argv[f], "-indentation")) {
                indentation = argv[f + 1];
                f++;
            }
        }
    }

    return email_fmt(soft, width, indentation);
}

