#!/usr/bin/env python

import sys, popen2, tempfile, os

class LINE:
   pass

# make this depend on an environment variable?
IDENTATION_CHARS = " \t>:;#%-"

TARGET_BLOCK_WIDTH = 70

def detect_identation_characters(line):
   """Returns number of identation chars minus trailing whitespace."""
   count = 0
   for f in line:
      if f in IDENTATION_CHARS: count = count + 1
      else: break
      
   return len(line[:count].rstrip()), count

def initial_scan(line):
   """Add attributes to the lines."""
   l = LINE()
   l.line = line
   l.total_length = len(line)
   l.ident, l.soft_ident = detect_identation_characters(line)
   l.length = l.total_length - l.soft_ident
   return l

def same_identation_lines(lines, soft_identation = 0):
   """Returns number of lines with same identation level."""
   f = 1
   while f < len(lines):
      if soft_identation and lines[f].soft_ident != lines[0].soft_ident:
         break
      if not soft_identation and lines[f].ident != lines[0].ident:
         break

      f = f + 1
         
   return f

def reformat_lines(lines, temp_filename, soft_identation = 0):
   """Dumps lines in list to file, the reads fmt's output."""
   assert lines
   # Find out the indentation level.
   if soft_identation:
      ident = lines[0].soft_ident
   else:
      ident = lines[0].ident

   # Depending on the level, store for later the string padding.
   if ident:
      padding = "%s " % lines[0].line[:ident]
      if soft_identation:
         padding = padding[:-1]
   else:
      padding = ""

   # Calculate length we will pass to the fmt command.
   length = max(TARGET_BLOCK_WIDTH - ident, 20)

   # create temporary file with content to ident
   file = open(temp_filename, "wt")
   num = 0
   while num < len(lines):
      string = lines[num].line[ident:]
      stripped = string.lstrip()
      if stripped:   file.write(stripped)
      else:          file.write(string)
      num += 1
   file.close()

   # call external tool and read it's stdout
   #sys.stderr.write("Calling with %d\n" % length);
   stdout, stdin = popen2.popen2 (["fmt", "-w", "%d" % length,
      "-u", temp_filename])
   stdin.close()
   new_lines = stdout.readlines()
   stdout.close()
   os.unlink(temp_filename)

   # output lines, taking care of last line's trailing '\n'
   for f in range(len(new_lines)-1):
      sys.stdout.write("%s%s" % (padding, new_lines[f]))
   # remove additional trailing \n if it wasn't originally present
   if lines[-1].line.find("\n") >= 0:
      sys.stdout.write("%s%s" % (padding, new_lines[-1:][0]))
   else:
      sys.stdout.write("%s%s" % (padding, new_lines[-1:][0][:-1]))

def main():
   soft = "-soft" in sys.argv[1:]
   lines = map(initial_scan, sys.stdin.readlines())
   temp_filename = tempfile.mktemp(".email-fmt.tmp")

   f = 0
   while f < len(lines):
      look = same_identation_lines(lines[f:], soft)
      reformat_lines(lines[f:f+look], temp_filename, soft)
      f = f + look
 
if __name__ == "__main__":
   main()
      
